#include <PS2Keyboard.h>
#include <SPI.h>
#include <Adafruit_GFX.h>
#include <Adafruit_ILI9341.h>

// Define the pins
#define PIN_CLK 13
#define PIN_MOSI 11
#define PIN_RES 8
#define PIN_DC 9
#define PIN_CS1 10 // Assuming CS1 is used for Chip Select

const int DataPin = 4;
const int IRQpin = 3;

PS2Keyboard keyboard;
Adafruit_ILI9341 display = Adafruit_ILI9341(PIN_CS1, PIN_DC, PIN_RES);

// Screen dimensions
const int screenWidth = 240;
const int screenHeight = 320;

// Box dimensions
const int boxWidth = 30;
const int boxHeight = 35;
const int boxSpacing = 5;

// Calculate starting x and y positions to center the grid
const int gridWidth = 5 * boxWidth + 4 * boxSpacing;
const int gridHeight = 6 * boxHeight + 5 * boxSpacing;
const int gridStartX = (screenWidth - gridWidth) / 2;
const int gridStartY = (screenHeight - gridHeight) / 2;

// Define the list of possible target words (100 five-letter words)
const char* wordList[100] = {
  "APPLE", "BREAD", "CRANE", "DREAM", "EAGLE", "FAITH", "GRAIN", "HOUSE", "IRISH", "JUMPY",
  "KHAKI", "LEMON", "MANGO", "NOBLE", "OCEAN", "PAINT", "QUERY", "RIVER", "SUGAR", "THUMB",
  "UNION", "VIVID", "WATER", "XENON", "YEARN", "ZEBRA", "ACTOR", "BANJO", "CHAIR", "DODGE",
  "EARTH", "FABLE", "GHOST", "HONEY", "INPUT", "JAZZY", "KNACK", "LASER", "MAGIC", "NEEDY",
  "OPERA", "PRISM", "QUICK", "ROVER", "STEAM", "TOUCH", "UPPER", "VOICE", "WHISK", "XYLEM",
  "YOUNG", "ZONED", "ANGEL", "BRAVE", "CLOVE", "DOUBT", "EXACT", "FLEET", "GRACE", "HORSE",
  "INDEX", "JAUNT", "KAYAK", "LIGHT", "MIRTH", "NIGHT", "OPINE", "PLUMB", "QUEST", "ROUTE",
  "STORY", "TRUTH", "USAGE", "VAULT", "WORTH", "XENIA", "YIELD", "ZAPPY", "AGILE", "BLAST",
  "CHILL", "DRIFT", "ELATE", "FROZE", "GIANT", "HUMAN", "IMAGE", "JOKER", "KNEEL", "LODGE"
};

const int wordCount = 100;
const int maxGuesses = 6;
char guess[6] = {0}; // buffer for the current guess
int charIndex = 0; // current character index for the guess
int currentGuess = 0; // current guess number
bool gameOver = false;
const char* targetWord;

// Menu options
const char* menuOptions[] = {"Rules", "Play"};
const int menuOptionCount = sizeof(menuOptions) / sizeof(menuOptions[0]);
int currentOption = 0;

const int menuX = 10;
const int menuY = 100;
const int menuSpacing = 20;
const int boxWidthMenu = 35; // Width of each rectangle
const int boxHeightMenu = 35; // Height of each rectangle
const int titleSpacing = 50; // Spacing between title and menu options
// Rules text
const char* rulesText = "Wordle Rules:\n\n1. Guess the hidden word.\n2. You have 6 attempts.\n3. Each guess will be displayed.\n4. Green box: correct letter and pos.\n5. Yellow box: correct letter, wrong pos.\n6. Grey box: incorrect letter.\n\n Press BACKSPACE to return to main menu.";

void resetGame() {
  targetWord = wordList[random(0, wordCount)];
  currentGuess = 0;
  charIndex = 0;
  memset(guess, 0, sizeof(guess));
  gameOver = false;

  // Clear the display and redraw the grid
  display.fillScreen(ILI9341_BLACK);
  display.setTextColor(ILI9341_WHITE);
  display.setTextSize(2);

  for (int i = 0; i < maxGuesses; i++) {
    for (int j = 0; j < 5; j++) {
      display.drawRect(gridStartX + j * (boxWidth + boxSpacing), gridStartY + i * (boxHeight + boxSpacing), boxWidth, boxHeight, ILI9341_WHITE);
    }
  }
}

void drawMenu() {
  display.fillScreen(ILI9341_BLACK);

  // Draw "WORDLE" title
  display.setTextSize(4);
  display.setTextColor(ILI9341_WHITE);
  for (int i = 0; i < 6; i++) {
    display.fillRect(menuX + i * (boxWidth + 4), menuY - titleSpacing, boxWidthMenu, boxHeightMenu, ILI9341_GREEN);
    display.setCursor(menuX + i * (boxWidth + 4) + 4, menuY - titleSpacing + 8);
    display.print("WORDLE"[i]);
  }

  // Draw menu options
  display.setTextSize(2);
  display.setTextColor(ILI9341_WHITE);
  display.setCursor(menuX, menuY + titleSpacing);
  for (int i = 0; i < menuOptionCount; i++) {
    if (i == currentOption) {
      display.setTextColor(ILI9341_GREEN); // Highlight current option
    } else {
      display.setTextColor(ILI9341_WHITE);
    }
    display.println(menuOptions[i]);
    display.setCursor(menuX, menuY + titleSpacing + (i + 1) * menuSpacing);
  }
}


void showRulesPopup() {
  // Clear the display
  display.fillScreen(ILI9341_BLACK);

  // Display the rules text
  display.setCursor(10, 10);
  display.setTextColor(ILI9341_WHITE);
  display.setTextSize(2);
  display.print(rulesText);

  // Wait for PS2_BACKSPACE key press to return to the main menu
  while (true) {
    if (keyboard.available()) {
      char c = keyboard.read();
      if (c == PS2_BACKSPACE) {
        drawMenu();
        break; // Return to the main menu
      }
    }
  }
}



void showPopup(const char* message, const char* subMessage = nullptr) {
  int popupWidth = 180;
  int popupHeight = 60;
  int popupX = (screenWidth - popupWidth) / 2;
  int popupY = (screenHeight - popupHeight) / 2;

  // Draw a pop-up box
  display.fillRect(popupX, popupY, popupWidth, popupHeight, ILI9341_WHITE);
  display.drawRect(popupX, popupY, popupWidth, popupHeight, ILI9341_BLACK);

  // Display the message
  display.setCursor(popupX + 10, popupY + 10);
  display.setTextColor(ILI9341_BLACK);
  display.setTextSize(2);
  display.print(message);

  if (subMessage != "") {
    display.setCursor(popupX + 10, popupY + 30);
    display.setTextSize(1);
    display.print(subMessage);
    display.setCursor(popupX + 10, popupY + 45);
    display.setTextSize(1);
    display.print("Press ESC to start again");
  }
  else
  {
    display.setCursor(popupX + 10, popupY + 30);
    display.setTextSize(1);
    display.print("Press ESC to start again");
  }

}


void startWordleGame() {
  // Reset the game
  resetGame();

  // Game loop
  while (!gameOver) {
    if (keyboard.available()) {
      char c = keyboard.read();

      if (c >= 'a' && c <= 'z') {
        c = c - 'a' + 'A'; // convert to uppercase
      }

      if (charIndex < 5 && c >= 'A' && c <= 'Z') {
        guess[charIndex] = c;
        display.setCursor(gridStartX + charIndex * (boxWidth + boxSpacing) + 8, gridStartY + currentGuess * (boxHeight + boxSpacing) + 8);
        display.print(c);
        charIndex++;
      }

      if (c == PS2_ENTER && charIndex == 5) {
        guess[5] = '\0'; // Null-terminate the guess

        // Determine box colors based on the guess
        for (int i = 0; i < 5; i++) {
          if (guess[i] == targetWord[i]) {
            display.fillRect(gridStartX + i * (boxWidth + boxSpacing), gridStartY + currentGuess * (boxHeight + boxSpacing), boxWidth, boxHeight, ILI9341_GREEN);
          } else if (strchr(targetWord, guess[i])) {
            display.fillRect(gridStartX + i * (boxWidth + boxSpacing), gridStartY + currentGuess * (boxHeight + boxSpacing), boxWidth, boxHeight, ILI9341_YELLOW);
          } else {
            display.fillRect(gridStartX + i * (boxWidth + boxSpacing), gridStartY + currentGuess * (boxHeight + boxSpacing), boxWidth, boxHeight, ILI9341_DARKGREY);
          }
          display.setCursor(gridStartX + i * (boxWidth + boxSpacing) + 8, gridStartY + currentGuess * (boxHeight + boxSpacing) + 8);
          display.setTextColor(ILI9341_WHITE);
          display.print(guess[i]);
        }

        if (strcmp(guess, targetWord) == 0) {
          showPopup("You Win!", "");
          gameOver = true;
          return;
        }

        currentGuess++;
        charIndex = 0;
        memset(guess, 0, sizeof(guess));

        if (currentGuess == maxGuesses) {
          char buffer[30];
          sprintf(buffer, "The word is: %s", targetWord);
          showPopup("You lose!", buffer);
          gameOver = true;
          return;
        }
      }

      if (c == PS2_BACKSPACE && charIndex > 0) {
        charIndex--;
        guess[charIndex] = '\0';
        display.fillRect(gridStartX + charIndex * (boxWidth + boxSpacing), gridStartY + currentGuess * (boxHeight + boxSpacing), boxWidth, boxHeight, ILI9341_BLACK);
        display.drawRect(gridStartX + charIndex * (boxWidth + boxSpacing), gridStartY + currentGuess * (boxHeight + boxSpacing), boxWidth, boxHeight, ILI9341_WHITE);
      }
    }
  }
}


void setup() {
  delay(1000);
  keyboard.begin(DataPin, IRQpin);
  Serial.begin(9600);
  Serial.println("Keyboard Test:");

  display.begin();
  display.setRotation(0);
  drawMenu();
}

void loop() {
  if (keyboard.available()) {
    char c = keyboard.read();

    if (!gameOver) {
      if (currentOption == 1 && c == PS2_ENTER) {
        startWordleGame();
        return;
      }
    }
    if (gameOver) {
      if (c == PS2_ESC) {
        resetGame();
        drawMenu();
      }
    return;
    }
    if (c == PS2_UPARROW) {
      currentOption--;
      if (currentOption < 0) {
        currentOption = menuOptionCount - 1;
      }
      drawMenu();
    } else if (c == PS2_DOWNARROW) {
      currentOption++;
      if (currentOption >= menuOptionCount) {
        currentOption = 0;
      }
      drawMenu();
    } else if (c == PS2_ENTER) {
      if (currentOption == 0) {
        showRulesPopup();
      }
      }
  }
}
